/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16215 Alton Parkway
*             P.O. Box 57013
*             Irvine, California 92619-7013
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: asn1_k_oid.c
*
****************************************************************************
*    Description:
*
*       additional implementation of packetCable PKINIT request and reply
*
****************************************************************************/

/*
 * Abstract Syntax Notation One, ASN.1
 * As defined in ISO/IS 8824 and ISO/IS 8825
 * This implements a subset of the above International Standards that
 * is sufficient to implement SNMP.
 *
 * Encodes abstract data types into a machine independent stream of bytes.
 *
 */
/**********************************************************************
  Copyright 1988, 1989, 1991, 1992 by Carnegie Mellon University

  All Rights Reserved

  Permission to use, copy, modify, and distribute this software and its
  documentation for any purpose and without fee is hereby granted,
  provided that the above copyright notice appear in all copies and that
  both that copyright notice and this permission notice appear in
  supporting documentation, and that the name of CMU not be
  used in advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
  CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
  ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
  SOFTWARE.
 ******************************************************************/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "asn1_k_oid.h"
#include "seckrb.h"
#include "ParseLibMain.h"


/*
 * call after mta_asn_build_header to verify result.
 */
static
int _asn_build_header_check(const char *str, unsigned char *data,
      int datalen, int typedlen)
{
   char ebuf[128];

   if (data == NULL)
   {
      /* error message is set */
      return 1;
   }
   if (datalen < typedlen)
   {
      if (gprintFlag)
	  {
         printf(ebuf, "%s: bad header, length too short: %d < %d", str, datalen, typedlen);
      }
      return 1;
   }
   return 0;
}



/*
 * mta_asn_parse_length - interprets the length of the current object.
 *  On exit, length contains the value of this length field.
 *
 *  Returns a pointer to the first byte after this length
 *  field (aka: the start of the data field).
 *  Returns NULL on any error.

 unsigned char * mta_asn_parse_length(
 unsigned char     *data         IN - pointer to start of length field
 unsigned long     *length       OUT - value of length field
 */
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//TRC change the name of this function!!!!!!
//NetSnmp has a function called asn_parse_length
//
unsigned char *
mta_asn_parse_length(unsigned char  *data,
      unsigned long  *length)
{
   const char *errpre = "parse length";
   char ebuf[128];
   unsigned char lengthbyte;

   if (!data || !length)
   {
      if (gprintFlag)
	  {
         printf("parse length: NULL pointer");
	  }
      return NULL;
   }
   lengthbyte = *data;

   if (lengthbyte & ASN_LONG_LEN)
   {
      lengthbyte &= ~ASN_LONG_LEN;	/* turn MSb off */
      if (lengthbyte == 0)
      {
         if (gprintFlag)
         {
            printf(ebuf, "%s: indefinite length not supported", errpre);
		 }
         return NULL;
      }
      if (lengthbyte > sizeof(long))
      {
         if (gprintFlag)
		 {
            printf(ebuf, "%s: data length %d > %d not supported", errpre,
               lengthbyte, sizeof(long));
		 }
         return NULL;
      }
      data++;
      *length = 0;  /* protect against short lengths */
      while(lengthbyte--)
      {
         *length <<= 8;
         *length |= *data++;
      }
      return data;
   }
   else
   { /* short asnlength */
      *length = (long)lengthbyte;
      return data + 1;
   }
}


/*
 * in: data[] - encoded oid
 * in: datalength - length of encoded oid
 * out: objid[] - array of long returned with decoded oid
 * out: objidlength - length of array of longs
*/
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//TRC change the name of this function!!!!!!
//NetSnmp has a function called asn_parse_objid
//
int mta_asn_parse_objid(unsigned char *data, int *datalength,
      unsigned long *objid, int *objidlength)
{
   /*
    * ASN.1 objid ::= 0x06 asnlength subidentifier {subidentifier}*
    * subidentifier ::= {leadingbyte}* lastbyte
    * leadingbyte ::= 1 7bitvalue
    * lastbyte ::= 0 7bitvalue
    */
   unsigned char   *buf_p=data;
   unsigned long   subidentifier;
   long   length;
   unsigned long   asn_length;
   int i, j;

   /* printf("enter mta_asn_parse_objid datalength=%d, objidlength=%d.\n", *datalength, *objidlength); */
   /* Handle invalid object identifier encodings of the form 06 00 robustly */
   asn_length = *datalength;
   if (asn_length == 0)
      objid[0] = objid[1] = 0;

   length = asn_length;

   i = 0;
   j=0;
   while (length > 0)
   {
      subidentifier = 0;
      do
      {	/* shift and add in low order 7 bits */
         subidentifier = (subidentifier << 7) + (*buf_p & ~ASN_BIT8);
         length--;
      } while (*buf_p++ & ASN_BIT8);
      /*?? note, this test will never be true, since the largest value
        of subidentifier is the value of MAX_SUBID! */
      if (subidentifier > MAX_SUBID)
      {
         if (gprintFlag)
		 {
            printf("subidentifier too large\n");
		 }
         return 0;
      }
      i++;
      objid[i] = (unsigned long)subidentifier;
   }
   *objidlength = i+1;


   /*
    * The first two subidentifiers are encoded into the first component
    * with the value (X * 40) + Y, where:
    *	X is the value of the first subidentifier.
    *  Y is the value of the second subidentifier.
    */
   subidentifier = (unsigned char)objid[1];
   if (subidentifier == 0x2B)
   {
      objid[0] = 1;
      objid[1] = 3;
   }
   else
   {
      if (subidentifier < 40)
      {
         objid[0] = 0;
         objid[1] = subidentifier;
      }
      else if (subidentifier < 80)
      {
         objid[0] = 1;
         objid[1] = subidentifier - 40;
      }
      else if (subidentifier < 120)
      {
         objid[0] = 2;
         objid[1] = subidentifier - 80;
      }
      else
      {
         objid[1] = (subidentifier % 40);
         objid[0] = ((subidentifier - objid[1]) / 40);
      }
   }
   /* printf("exit mta_asn_parse_objid datalength=%d, objidlength=%d.\n", *datalength, *objidlength); */
   return 0;
}

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//TRC change the name of this function!!!!!!
//NetSnmp has a function called asn_build_objid
//
int mta_asn_build_objid(mta_oid *objid, int objidlength,  unsigned char *out_data,
      int *datalength)
{
   /*
    * ASN.1 objid ::= 0x06 asnlength subidentifier {subidentifier}*
    * subidentifier ::= {leadingbyte}* lastbyte
    * leadingbyte ::= 1 7bitvalue
    * lastbyte ::= 0 7bitvalue
    */
   int asnlength;
   mta_oid *op = objid;
   unsigned char objid_size[MAX_OID_LEN];
   unsigned long objid_val;
   unsigned char first_objid_val;
   int i,j;
   unsigned char *data=out_data;



   /* printf("mta_asn_build_objid objidlength=%d, datalength=%d.\n", objidlength, *datalength); */
   /* check if there are at least 2 sub-identifiers */
   if (objidlength == 0)
   {
      /* there are not, so make OID have two with value of zero */
      objid_val = 0;
      objidlength = 2;
   }
   else if (objidlength == 1)
   {
      /* encode the first value */
      objid_val = (op[0] * 40);
      objidlength = 2;
      op++;
   }
   else
   {
      /* combine the first two values */
      if ( op[1] > 40 )
      {
         if (gprintFlag)
		 {
            printf("build objid: bad second subidentifier");
		 }
         return NULL;
      }
      objid_val = (op[0] * 40) + op[1];
      op += 2;
   }
   first_objid_val = objid_val;

   /* calculate the number of bytes needed to store the encoded value */
   for (i = 1, asnlength = 0;;)
   {
      if (objid_val < (unsigned)0x80)
      {
         objid_size[i] = 1;
         asnlength += 1;
      }
      else if (objid_val < (unsigned)0x4000)
      {
         objid_size[i] = 2;
         asnlength += 2;
      }

      else if (objid_val < (unsigned)0x200000)
      {
         objid_size[i] = 3;
         asnlength += 3;
      }
      else if (objid_val < (unsigned)0x10000000)
      {
         objid_size[i] = 4;
         asnlength += 4;
      }
      else
      {
         objid_size[i] = 5;
         asnlength += 5;
      }
      i++;
      if (i >= (int)objidlength)
         break;
      objid_val = *op++;
   }

   /* store the ASN.1 tag and length */
   /*
   data = mta_asn_build_header(data, datalength, type, asnlength);
   if (_asn_build_header_check("build objid", data, *datalength, asnlength))
      return NULL;
   */

   /* store the encoded OID value */
   for (i = 1, objid_val = first_objid_val, op = objid+2;
         i < (int)objidlength;
         i++)
   {
      if (i != 1) objid_val = *op++;
      switch (objid_size[i])
      {
         case 1:
            *data++ = (unsigned char)objid_val;
            break;

         case 2:
            *data++ = (unsigned char)((objid_val>>7) | 0x80);
            *data++ = (unsigned char)(objid_val & 0x07f);
            break;

         case 3:
            *data++ = (unsigned char)((objid_val>>14) | 0x80);
            *data++ = (unsigned char)((objid_val>>7 & 0x7f) | 0x80);
            *data++ = (unsigned char)(objid_val & 0x07f);
            break;

         case 4:
            *data++ = (unsigned char)((objid_val>>21) | 0x80);
            *data++ = (unsigned char)((objid_val>>14 & 0x7f) | 0x80);
            *data++ = (unsigned char)((objid_val>>7 & 0x7f) | 0x80);
            *data++ = (unsigned char)(objid_val & 0x07f);
            break;

         case 5:
            *data++ = (unsigned char)((objid_val>>28) | 0x80);
            *data++ = (unsigned char)((objid_val>>21 & 0x7f) | 0x80);
            *data++ = (unsigned char)((objid_val>>14 & 0x7f) | 0x80);
            *data++ = (unsigned char)((objid_val>>7 & 0x7f) | 0x80);
            *data++ = (unsigned char)(objid_val & 0x07f);
            break;
      }
   }

   /* return the length and data ptr */
   *datalength = asnlength;

   /* printf("mta_asn_build_objid objidlength=%d, datalength=%d.\n", objidlength, *datalength); */
   return 0;
}

