/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: bcmos.h
*    Creation Date: 8 June 2000 (v0.00)
*    VSS Info:
*        $Revision: 1.1 $
*        $Date: 2002/08/21 17:53:19 $
*
****************************************************************************
*    Description:
*
*      Broadcom Generic Operating System Functions.
*
****************************************************************************/

#ifndef _BCMOS_H
#define _BCMOS_H

#ifdef __cplusplus
extern "C" {
#endif

#include <hausenv.h>
#include <bcmosCfg.h>

/*
*****************************************************************************
** TYPE DECLARATIONS
*****************************************************************************
*/

/*****************************************************************************
** OS PRIORITIES
******************************************************************************/

/* This define the different 'task classes' that exist for the MTA. Tasks of
 * the same class will be assigned an equal priority level. The absolute
 * priority assigned to each task class are given to the BCMOS during
 * initialization (bcmOsInit). This interface supplies the priorities
 * that the various MTA classes should use in order to be compatible
 * with the overall system architecture. */
typedef enum
{
   OS_TASK_RTP_DATA     = -1, /* RTP voice data tasks */
   OS_TASK_DATA         = -2, /* Non-voice data tasks, e.g. NCS parsing */
   OS_TASK_MANAGEMENT   = -3, /* MTA management task, e.g. DHCP, DNS, provisioning */
   OS_TASK_HARDWARE     = -4, /* MTA hardware level task */
   OS_TASK_BACKGROUND   = -5, /* MTA idle background task */
   OS_TASK_ROOT         = -6, /* MTA root application task */
   OS_TASK_DEFAULT      = -7, /* Priority of tasks not otherwise specified above */

   OS_TASK_CLASS_MAX    = 7
} OS_TASK_CLASS;


/* Priority level structure for EMTA threads. This structure contains absolute task
 * priority levels for various classes of EMTA tasks. The absolute task priority
 * levels are supplied to BCMOS when it is initialized. This allows the task
 * levels to be set by an external entity that understands how they should be
 * set in order to be compatible with the overall system architecture */
typedef struct OS_THREAD_CLASS_PRIO_
{
   UINT32 priorityLevel[ OS_TASK_CLASS_MAX ];

} OS_THREAD_CLASS_PRIO;


/* Message queue element that is sent or received to/from the queue */
typedef struct OS_QMSG
{
   UINT32 buffer[4];
} OS_QMSG;


/* Define generic OS variable types */
typedef unsigned long      OS_SEMID;      /* Semaphore ID type */
typedef unsigned long      OS_MUTEXID;    /* Mutex ID type */
typedef unsigned long      OS_LOCKKEY;    /* Lock/unlock parameter type */
typedef unsigned long      OS_TICKS;      /* Tick type */
typedef unsigned long      OS_TASKID;     /* Task ID type */
typedef unsigned long      OS_EVENT;      /* Event object */
typedef unsigned long      OS_EVENTSET;   /* Event set object */
typedef unsigned long      OS_EVENTFLG;   /* Event processing attributes */
typedef unsigned long      OS_TASKARG;    /* Task start argument */
typedef unsigned long      OS_TIMERID;    /* Timer ID */
typedef unsigned long      OS_QID;        /* Message Queue ID */
typedef unsigned long      OS_QSIZE;      /* Message Queue Size */
typedef unsigned long      OS_TMDATE;     /* Date: Year, Month, Daty */
typedef unsigned long      OS_TMTIME;     /* Time: Hour, minute, Second */
typedef unsigned long      OS_FD_SET;     /* File descriptor set */


typedef struct OS_IN_ADDR
{
   unsigned long S_addr;
} OS_IN_ADDR;

/* Socket address, internet style */
typedef struct OS_SOCKADDR
{
   short          sin_family;    /* must be AF_INET */
   unsigned short sin_port;      /* 16-bit port number */
   OS_IN_ADDR     sin_addr;      /* 32-bit IP address */
   long           sin_ifno;      /* 32-bit interface number */
   char           sin_zero[4];   /* must be 0 */
} OS_SOCKADDR;


/* Message queue type: tasks can wait for messages either by task priority or strictly FIFO */
typedef enum
{
   OS_QUEUE_FIFO,
   OS_QUEUE_PRIORITY
} OS_QUEUE_TYPE;


/* BCMOS API return codes */
typedef enum
{
   OS_STATUS_OK = 0,
   OS_STATUS_TIMEOUT,
   OS_STATUS_ERR,

   /* Insert new status codes here */

   /* Application level return codes for modules which "inherit" from the BCMOS */
   OS_STATUS_APPLEVEL1,
   OS_STATUS_APPLEVEL2,
   OS_STATUS_APPLEVEL3,
   OS_STATUS_APPLEVEL4,

   OS_STATUS_MAX
} OS_STATUS;


/*
*****************************************************************************
** CONSTANT DECLARATIONS
*****************************************************************************
*/

/* Convert from OS_TASK enumerated type to an index. This can be used to
 * index the array of priority levels contained in the OS_THREAD_CLASS_PRIO
 * structure. (Convert from negative to positive and 1-reference to 0-reference
 * index). */
#define OS_TASK_TO_INDEX( a ) ( -(a) - 1 )


/*
** NULL file-descriptor set value. Used with bcmOsSelect() to indicate that
** a set is not provided.
*/
#define OS_FD_SET_NULL  0xffffffff


/*
** Definition of infinite time constant
*/
#define OS_WAIT_FOREVER  (OS_TICKS) (-1)

/*
** DNS specific defines
*/
#define  OS_DNS_MAX_SERVERS      4     /* Maximum number of DNS servers supported */


/* Macros to deal with seconds and ticks. The value of OS_MSEC_PER_TICK is arbitrary. It
 * is OS independent and does not correspond to any real OS tick value. Real OS
 * tick values are encapsulated by the bcmOS layer implementation. */
#define OS_MSEC_PER_TICK   1
#define SEC2TICKS(sec)   ( (sec * 1000) / OS_MSEC_PER_TICK)   /* Convert Seconds to Ticks */
#define TICKS2SEC(ticks) ( (ticks * OS_MSEC_PER_TICK) / 1000) /* Convert Ticks to Seconds */

#define TWO_MSEC           ((OS_TICKS) (2/OS_MSEC_PER_TICK))
#define TEN_MSEC           ((OS_TICKS) (10/OS_MSEC_PER_TICK))
#define SEVENTEEN_MSEC     ((OS_TICKS) (17/OS_MSEC_PER_TICK))
#define TWENTY_MSEC        ((OS_TICKS) (20/OS_MSEC_PER_TICK))
#define FIFTY_MSEC         ((OS_TICKS) (50/OS_MSEC_PER_TICK))
#define ONE_HUNDRED_MSEC   ((OS_TICKS) (100/OS_MSEC_PER_TICK))
#define FIVE_HUNDRED_MSEC  ((OS_TICKS) (500/OS_MSEC_PER_TICK))
#define ONE_SECOND         ((OS_TICKS) (1000/OS_MSEC_PER_TICK))
#define TWO_SECONDS        ((OS_TICKS) (2000/OS_MSEC_PER_TICK))

/***********************************************************************/
/* Definitions for Network Order Conversions Routines                  */
/***********************************************************************/
#ifndef htonl
   #if CPU_LITTLE_ENDIAN
      /*******************************************************************/
      /* Change byte order so the least significant byte is stored at    */
      /* the lowest address. Unsigned to prevent integer overflow on     */
      /* i960. Big endian cast unsigned for constant handling of vars.   */
      /*******************************************************************/
      #define htonl(addr)     ( (((unsigned long)(addr) & 0x000000FF)<<24) | \
                              (((unsigned long)(addr) & 0x0000FF00)<<8)  | \
                              (((unsigned long)(addr) & 0x00FF0000)>>8)  | \
                              (((unsigned long)(addr) & 0xFF000000)>>24))

      #define ntohl(addr)     htonl(addr)

      #define htons(addr)     (unsigned short)((((unsigned short)(addr) & 0x000000FF)<<8)  | \
                              (((unsigned short)(addr) & 0x0000FF00)>>8))

      #define ntohs(addr)     htons(addr)
   #elif CPU_BIG_ENDIAN
      #define htonl(a)       ((unsigned long)(a))
      #define ntohl(a)       ((unsigned long)(a))
      #define htons(a)       ((unsigned short)(a))
      #define ntohs(a)       ((unsigned short)(a))
   #elif
      #error "Unknown CPU"
   #endif
#endif   /* htonl */


/* BCMOS socket related constant definitions.  These constants are
 * currently defined as a set of enums since they are treated as
 * OS-independent constants.  Proper updates will be done later
 * to convert from OS-independent values to OS-dependent values */
typedef enum
{
   OS_AF_INET = 0,
   OS_SOCK_DGRAM,
   OS_SOL_SOCKET,
   OS_SO_REUSEADDR,
   OS_INADDR_ANY,
   OS_IPPROTO_IP,
   OS_IPPROTO_UDP,
   OS_IP_TOS,
   OS_SOCKET_OPT_MAX
} OS_SOCKOPT;

typedef BOOL (*OS_ABORT_FNC)(void);

/*****************************************************************************
** EXTERNAL FUNCTION PROTOTYPES
******************************************************************************/

/*****************************************************************************
** FUNCTION:   bcmOsLock
**
** PURPOSE:    Prevent the OS from switching tasks for concurrency protection
**
** PARAMETERS: lockKey - pointer to OS_LOCKKEY
**
** RETURNS:    OS_STATUS_OK
**
** NOTES:      The lockKey value is passed back to bcmOsUnlock to re-enable
**             the OS's scheduler
*****************************************************************************/
OS_STATUS bcmOsLock(OS_LOCKKEY *lockKey);


/*****************************************************************************
** FUNCTION:   bcmOsUnlock
**
** PURPOSE:    Re-enable the OS sheduler
**
** PARAMETERS: lockKey - input - value set by bcmOsLock
**
** RETURNS:    OS_STATUS_OK
**
** NOTES:
*****************************************************************************/
OS_STATUS bcmOsUnlock( OS_LOCKKEY lockKey );


/*****************************************************************************
** FUNCTION:   bcmOsInit
**
** PURPOSE:    initialize any os specific variables
**
** PARAMETERS: priorities - structure that contains the absolute priority
**                          levels associated with each class of MTA task
**
** RETURNS:    OS_STATUS_OK
*****************************************************************************/
OS_STATUS bcmOsInit( OS_THREAD_CLASS_PRIO *priorityLevels );


/*****************************************************************************
** FUNCTION:   bcmOsSleep
**
** PURPOSE:    Sleep for a specified number of ticks
**
** PARAMETERS: sleeptime - the time to sleep for (in ticks)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
*****************************************************************************/
OS_STATUS bcmOsSleep( OS_TICKS sleeptime );


/*****************************************************************************
** FUNCTION:   bcmOsGetTime
**
** PURPOSE:    get current system time in ticks.
**
** PARAMETERS: osTime - pointer to OS_TICKS (output)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       This function is only used to measure relative time for the
**             purposes of timing out syncronous requests
*****************************************************************************/
OS_STATUS bcmOsGetTime(OS_TICKS *osTime);

/*****************************************************************************
** FUNCTION:   bcmOsGetDeltaTime
**
** PURPOSE:    Get delta system time in ticks.
**
** PARAMETERS: prevTime - previous OS_TICKS value (input)
**             curTime - current OS_TICKS value (output)
**             deltaTime - ticks since 'prevTime' (output)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       This function is only used to measure relative time for the
**             purposes of timing out syncronous requests. It will handle
**             wrap around conditions.
*****************************************************************************/
OS_STATUS bcmOsGetDeltaTime(OS_TICKS prevTime, OS_TICKS *curTime, OS_TICKS *deltaTime);

/*****************************************************************************
** FUNCTION:   bcmOsGetTimeEx
**
** PURPOSE:    gets full time realted information (date, time, ticks).
**
** PARAMETERS: date  - returns the Date related bits (y, m, d)
**             time  - returns the Time related bits (h, m, s)
**             time  - returns ticks
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       VxWorks not supported
**
*****************************************************************************/
OS_STATUS bcmOsGetTimeEx(OS_TMDATE *date, OS_TMTIME *time, OS_TICKS *ticks);

/*****************************************************************************
** FUNCTION:   bcmOsSetTime
**
** PURPOSE:    Sets full time related information (date, time, ticks).
**
** PARAMETERS: date  - sets the Date related bits (y, m, d)
**             time  - sets the Time related bits (h, m, s)
**             time  - sets ticks
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       VxWorks not supported
**
*****************************************************************************/
OS_STATUS bcmOsSetTime(OS_TMDATE date, OS_TMTIME time, OS_TICKS ticks);


/*****************************************************************************
** FUNCTION:   bcmOsTaskCreate
**
** PURPOSE:    Create a task.
**
** PARAMETERS: name        - name of the of the task
**             stackSize   - task stack size
**             priority    - task priority. If <priority> is an OS_TASK_CLASS
**                           enumerated type, the real OS priority will be set
**                           according to the OS_THREAD_CLASS_PRIO structure
**                           passed to bcmOsInit. If <priority> is a positive
**                           number, (all OS_TASK_CLASS types are negative),
**                           this number will be used as the real OS priority
**             taskentry   - the task entry point
**             argument    - the task argument
**             taskId      - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsTaskCreate( char* name,
                           int stackSize,
                           OS_TASK_CLASS taskClass,
                           void* taskEntry,
                           OS_TASKARG argument,
                           OS_TASKID *taskId );



/*****************************************************************************
** FUNCTION:   bcmOsTaskDelete
**
** PURPOSE:    deletes a task
**
** PARAMETERS: taskID - task identification
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsTaskDelete( OS_TASKID taskId );


/*****************************************************************************
** FUNCTION:   bcmOsTaskRestart
**
** PURPOSE:    restarts a task
**
** PARAMETERS: name - name of the task
**             argument - the task arguments
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsTaskRestart( char* name, OS_TASKARG argument );



/*****************************************************************************
** FUNCTION:   bcmOsTaskIdent
**
** PURPOSE:    obtains the task identifier of the calling task
**
** PARAMETERS: taskName - name of task
**             taskID   - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsTaskIdent(char* taskName, OS_TASKID *taskId );

/*****************************************************************************
** FUNCTION:   bcmOsTaskIdentSelf
**
** PURPOSE:    obtains the task identifier of the calling task
**
** PARAMETERS:
**             taskID   - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsTaskIdentSelf(OS_TASKID *taskId );


/*****************************************************************************
** FUNCTION:   bcmOsTaskSuspend
**
** PURPOSE:    Suspends a task
**
** PARAMETERS: taskID - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsTaskSuspend( OS_TASKID taskId );


/*****************************************************************************
** FUNCTION:   bcmOsTaskResume
**
** PURPOSE:    Resumes a suspended a task
**
** PARAMETERS: taskID - task identifier
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsTaskResume( OS_TASKID taskId );


/*****************************************************************************
** FUNCTION:   bcmOsTaskGetPriority
**
** PURPOSE:    Gets priority of task
**
** PARAMETERS: taskID   - task identifier
**             priority - task priority
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsTaskGetPriority( OS_TASKID taskId, OS_TASK_CLASS *priority );


/*****************************************************************************
** FUNCTION:   bcmOsTaskExists
**
** PURPOSE:    Verifies whether a task exists or not
**
** PARAMETERS: taskID   - task identifier to be verified
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsTaskExists( OS_TASKID taskId );

/*****************************************************************************
** FUNCTION:   bcmOsSemCreate
**
** PURPOSE:    Create a binary semaphore
**
** PARAMETERS: semName - semaphore name
**             semId - pointer to the semaphore handle
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsSemCreate(char *semName, OS_SEMID *semId);

/*****************************************************************************
** FUNCTION:   bcmOsSemTake
**
** PURPOSE:    Take a binary semaphore
**
** PARAMETERS: semId - Handle of the semaphore
**             timeout_ticks - timeout in ticks
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       this function ASSERTs that the operation was successful
******************************************************************************/
OS_STATUS bcmOsSemTake(OS_SEMID semId, OS_TICKS timeout);


/*****************************************************************************
** FUNCTION:   bcmOsSemGive
**
** PURPOSE:    Give a semaphore
**
** PARAMETERS: semId - Handle of the semaphore
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsSemGive( OS_SEMID semId );


/*****************************************************************************
** FUNCTION:   bcmOsSemDelete
**
** PURPOSE:    deletes a semaphore
**
** PARAMETERS: semID - semaphore identification
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsSemDelete( OS_SEMID semId );

/*****************************************************************************
** FUNCTION:   bcmOsMutexCreate
**
** PURPOSE:    Create a mutex
**
** PARAMETERS: mutexName   - mutex name
**             mutexId     - pointer to the mutex handle
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsMutexCreate(char *MutexName, OS_MUTEXID *mutexId);

/*****************************************************************************
** FUNCTION:   bcmOsMutexLock
**
** PURPOSE:    Lock a mutex
**
** PARAMETERS: mutexId        - Handle of the mutex
**             timeout_ticks  - timeout in ticks
**
** RETURNS:    OS_STATUS_OK   - success
**             error code     - failure
**
** NOTE:       this function ASSERTs that the operation was successful
******************************************************************************/
OS_STATUS bcmOsMutexLock(OS_MUTEXID mutexId, OS_TICKS timeout);

/*****************************************************************************
** FUNCTION:   bcmOsMutexUnlock
**
** PURPOSE:    Unlock a mutex
**
** PARAMETERS: mutexId        - Handle of the mutex
**
** RETURNS:    OS_STATUS_OK   - success
**             error code     - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsMutexUnlock( OS_MUTEXID mutexId );

/*****************************************************************************
** FUNCTION:   bcmOsMutexDelete
**
** PURPOSE:    deletes a mutex
**
** PARAMETERS: mutexID - mutex identification
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsMutexDelete( OS_MUTEXID mutexId );

/*****************************************************************************
** FUNCTION:   bcmOsEventCreate
**
** PURPOSE:    Creates a synchronization event object
**
** PARAMETERS: event [OUT] - handle for created event
**             taskId [IN[ - target task identifier. The created event will be sent
**                      to this task when bcmOsEventSend is invoked.
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventCreate( OS_EVENT *event, OS_TASKID *taskId );

/*****************************************************************************
** FUNCTION:   bcmOsEventDelete
**
** PURPOSE:    Deletes a synchronization event object
**
** PARAMETERS: event [IN] - handle for event to be deleted
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventDelete( OS_EVENT *event );

/*****************************************************************************
** FUNCTION:   bcmOsEventSetCreate
**
** PURPOSE:    Creates a synchronization event set object. Event sets are
**             a collection of event objects (which are created by bcmOsEventCreate)
**
** PARAMETERS: evtSet [OUT] - handle for created event set
**             taskId [IN] - target task identifier. Events contained within the set
**                      will be sent to this task when bcmOsEventSetSend is invoked.
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetCreate( OS_EVENTSET *evtSet, OS_TASKID *taskId );

/*****************************************************************************
** FUNCTION:   bcmOsEventSetDelete
**
** PURPOSE:    Deletes a synchronization event set object
**
** PARAMETERS: event [IN] - handle for event set to be deleted
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetDelete( OS_EVENTSET *event );

/*****************************************************************************
** FUNCTION:   bcmOsEventSetAdd
**
** PURPOSE:    Adds an event to an event set
**
** PARAMETERS: evtSet [IN] - handle for event set
**             event [IN] - the event which is to added to the event set
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetAdd( OS_EVENTSET *evtSet, OS_EVENT *event );

/*****************************************************************************
** FUNCTION:   bcmOsEventSend
**
** PURPOSE:    Sends an event to a task
**
** PARAMETERS: event [IN] - event to send
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSend( OS_EVENT *event );

/*****************************************************************************
** FUNCTION:   bcmOsEventSetSend
**
** PURPOSE:    Sends a set of events to a task
**
** PARAMETERS: evtSet [IN] - event set to send
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetSend( OS_EVENTSET *evtSet );

/*****************************************************************************
** FUNCTION:   bcmOsEventReceive
**
** PURPOSE:    Wait for an event to be sent to the calling task
**
** PARAMETERS: events [IN] - event to wait for
**             timeout - event timeout delay (in ticks)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventReceive( OS_EVENT *event, OS_TICKS timeout );

/*****************************************************************************
** FUNCTION:   bcmOsEventSetReceive
**
** PURPOSE:    Wait for a set of events to be sent to the calling task
**
** PARAMETERS: evtSet [IN] - events to wait for
**             waitForAll - if TRUE wait for all event to occur before returing,
**                          else only wait for a single event to occur
**             timeout - event timeout delay (in ticks)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetReceive( OS_EVENTSET *evtSet, BOOL waitForAll, OS_TICKS timeout );

/*****************************************************************************
** FUNCTION:   bcmOsEventSetOccurred
**
** PURPOSE:    This function should be used by a task to determine if the specified
**             event has been triggered.  This is usually done immediately after a
**             call to bcmOsEventSetReceive() returns.
** PURPOSE:
**
** PARAMETERS: evtSet [IN] - event set handle
**             event [IN] - check is this event occurred
**             occurred [OUT] - TRUE if event occurred, else FALSE
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetOccurred( OS_EVENTSET *evtSet, OS_EVENT *event, BOOL *occurred );


/*****************************************************************************
** FUNCTION:   bcmOsQueueSend
**
** PURPOSE:    Sends a message to a specified ordinary Message Queue.
**
** PARAMETERS: qid      - Message Queue ID to send a message to.
**             qMsg     - Message Buffer
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       bcmOsQueueSend does not block on send.
******************************************************************************/
OS_STATUS bcmOsQueueSend(OS_QID qid, OS_QMSG *qMsg);

/*****************************************************************************
** FUNCTION:   bcmOsQueueReceive
**
** PURPOSE:    receives a messgae from a specified ordinary Message Queue
**
** PARAMETERS: qid      - Message Queue ID to send a message to.
**             timeout  - Timeout in clock ticks
**             qMsg     - Message Buffer
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE: If timeout is 0, bcmOsQueueReceive() will be non-blocking, i.e. it
**       will return even if there is no available message in the queue. If
**       timeout is not 0, then this function will block until a message is
**       posted to the queue or the timeout occurs, which ever comes first.
**
******************************************************************************/
OS_STATUS bcmOsQueueReceive(OS_QID qid, OS_TICKS timeout, OS_QMSG *qMsg);

/*****************************************************************************
** FUNCTION:   bcmOsQueueCreate
**
** PURPOSE:    creates an ordinary message queue of size (qSize).
**
** PARAMETERS: qName    - Message Queue Name.
**             qSize    - Message Queue Size (number of messages in the queue)
**             qType    - Message Queue type - fifo or priority
**
**             qid      - Message Queue ID (returned)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:       PSOS limits the size of the messages within a queue to 16 bytes,
**             VXWORKS has no limit.  The generic bcmOsQueueCreate
**             will limit the VXWORKS message size to be 16 bytes so that the
**             interface is consistent across different operating systems.
******************************************************************************/
OS_STATUS bcmOsQueueCreate(char* qName, OS_QSIZE qSize, OS_QUEUE_TYPE qType, OS_QID *qid);


/*****************************************************************************
** FUNCTION:   bcmOsQueueDelete
**
** PURPOSE:    Delete a message queue
**
** PARAMETERS: qid      - Message Queue ID
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsQueueDelete( OS_QID qid );


/*****************************************************************************
** FUNCTION:   bcmOsSocketShare
**
** PURPOSE:    allows a task access to a socket.
**
** PARAMETERS: socketDesc    - Socket Descriptor
**             taskID        - task identifier
**             newSocketDesc - task's socket descriptor (output)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketShare( int socketDesc, OS_TASKID taskID, int *newSocketDesc);


/*****************************************************************************
** FUNCTION:   bcmOsSocketCreate
**
** PURPOSE:    creates a new socket.
**
** PARAMETERS:
**             domain        - socket domain
**             type          - socket type carrier
**             protocol      - socket protocol type
**             descriptor    - socket descriptor
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketCreate (OS_SOCKOPT domain, OS_SOCKOPT type, int protocol, int *descriptor);


/*****************************************************************************
** FUNCTION:   bcmOsSocketBind
**
** PURPOSE:    Binds a name/address to a socket
**
** PARAMETERS:
**             descriptor    - socket descriptor
**             name          - address/name structure to be bound
**             namelen       - size of structure
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketBind ( int descriptor, OS_SOCKADDR *name, int namelen);


/*****************************************************************************
** FUNCTION:   bcmOsSocketReceiveFrom
**
** PURPOSE:    receive a message from a socket
**
** PARAMETERS:
**             socketDesc    - socket to receive from
**             bufp          - pointer to data buffer
**             buflen        - length of buffer
**             flags         - flags to underlying protocols
**             ifno           - specifies outgoing interface for message
**                               (0 for default interface)
**             fromp         - where to copy sender's address
**             pFromLen      - value/result length of from
**             numBytesRec   - number of bytes received.
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketReceiveFrom ( int socketDesc, char *bufp, int buflen,
                                   int flags, long ifno, OS_SOCKADDR *fromp, int *pFromLen,
                                   int *numBytesRec);


/*****************************************************************************
** FUNCTION:   bcmOsSocketSendTo
**
** PURPOSE:    send a message to a socket
**
** PARAMETERS:
**             socketDesc    - socket to receive from
**             bufp          - pointer to data buffer
**             buflen        - length of buffer
**             flags         - flags to underlying protocols
**             ifno           - specifies outgoing interface for message
**                               (0 for default interface)
**             top           - recipient's address
**             toLen         - length of to sockaddr
**             numBytesSent  - number of bytes sent
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketSendTo ( int socketDesc, char *bufp, int buflen, int flags,
                              long info, OS_SOCKADDR *top, int toLen, int *numBytesSent );


/*****************************************************************************
** FUNCTION:   bcmOsSocketSetSocketOptions
**
** PURPOSE:    set socket options
**
** PARAMETERS:
**             socketDesc    - target socket
**             level         - protocol level of option
**             optname       - option name
**             optval        - pointer to option value
**             optlen        - option length
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketSetOptions ( int socketDesc, OS_SOCKOPT level, OS_SOCKOPT optname,
                                  char * optval, int optlen );


/*****************************************************************************
** FUNCTION:   bcmOsSocketClose
**
** PURPOSE:    close socket
**
** PARAMETERS:
**             socketDesc    - target socket
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketClose ( int socketDesc );


/*****************************************************************************
** FUNCTION:   bcmOsSocketCloseShared
**
** PURPOSE:    Close all sockets that were created by other tasks, but shared
**             with the calling task.
**
** PARAMETERS: none
**
** RETURNS:    OS_STATUS_OK  - success
**             error code    - failure
**
** NOTE:
**
******************************************************************************/
OS_STATUS bcmOsSocketCloseShared ( void );


/*****************************************************************************
** FUNCTION:   bcmOsGetOwnIpAddress()
**
** PURPOSE:    Retrieves the IP address of a given IP stack number (interface)
**
** PARAMETERS: netIface [IN]  - IP Interface Number
**             ipAddr [OUT]   - IP address corresponding to 'netIface'
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsGetOwnIpAddress( int netIface, IPADDR *ipAddr );


/*****************************************************************************
** FUNCTION:   bcmOsSelect()
**
** PURPOSE:    This function is used to multiplex I/O requests among multiple sockets.
**             Three sets of socket descriptors may be specified: a set of sockets from
**             which to read, a set to which to write and a set that may have pending
**             exceptional conditions.
**
**             If bcmOsSelect() returns successfully, the three sets indicate which socket
**             descriptors can be read, which can be written to, or which have exceptional
**             conditions pending. A timeout value may be specified.
**
** PARAMETERS: readset     [IN] - Set of sockets from which to read
**             writeset    [IN] - Set of sockets from which to write
**             exceptset   [IN] - Set of sockets that may have an exceptional condition pending
**             timeout     [IN] - Specifies a timeout option. If the timeout is set to 0,
**                                bcmOsSelect() returns immediately. If the timeout is
**                                OS_WAIT_FOREVER, bcmOsSelect() blocks until a descriptor is
**                                selectable.
**             numDesc     [OUT] - On success a nonzero value is returned that indicates the
**                                 number of descriptors on which events have occurred.
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsSelect( OS_FD_SET readset, OS_FD_SET writeset, OS_FD_SET exceptset, OS_TICKS timeout, int *numDesc );


/*****************************************************************************
** FUNCTION:   bcmOsFdSetOpen()
**
** PURPOSE:    Create a new file-descriptor set
**
** PARAMETERS: set [OUT]  - descriptor for created file-descriptor set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdSetOpen( OS_FD_SET *set );


/*****************************************************************************
** FUNCTION:   bcmOsFdSetClose()
**
** PURPOSE:    Close an existing file-descriptor set
**
** PARAMETERS: set [IN]  - descriptor for file-descriptor set to close
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdSetClose( OS_FD_SET set );


/*****************************************************************************
** FUNCTION:   bcmOsFdSetCopy()
**
** PURPOSE:    Copy a file-descriptor set
**
** PARAMETERS: dstSet [IN]  - descriptor for file-descriptor set to copy to
**             srcSet [IN]  - descriptor for file-descriptor set to copy from
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdSetCopy( OS_FD_SET dstSet, OS_FD_SET srcSet );


/*****************************************************************************
** FUNCTION:   bcmOsFdSet()
**
** PURPOSE:    Add a socket descriptor to a set. Emulates Berkeley Socket
**             macro FD_SET
**
** PARAMETERS: desc  [IN] - socket descriptor
**             set   [IN] - socket descriptor set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdSet( int desc, OS_FD_SET set );


/*****************************************************************************
** FUNCTION:   bcmOsFdClr()
**
** PURPOSE:    Remove a socket descriptor From a set. Emulates Berkeley Socket
**             macro FD_CLR
**
** PARAMETERS: desc  [IN] - socket descriptor
**             set   [IN] - socket descriptor set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdClr( int desc, OS_FD_SET set );


/*****************************************************************************
** FUNCTION:   bcmOsFdIsSet()
**
** PURPOSE:    Determines if a socket descriptor is a member of a set. Emulates
**             Berkeley Socket macro FD_ISSET
**
** PARAMETERS: desc  [IN] - socket descriptor
**             set   [IN] - socket descriptor set
 *             isSet [OUT] - indicates whether socket is member of the set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdIsSet( int desc, OS_FD_SET set, BOOL *isSet );


/*****************************************************************************
** FUNCTION:   bcmOsFdZero()
**
** PURPOSE:    Clears an entire socket descriptor set. Emulates Berkeley Socket
**             macro FD_ZERO
**
** PARAMETERS: set   [IN] - socket descriptor set
**
** RETURNS:   OS_STATUS_OK or OS_STATUS_ERR
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsFdZero( OS_FD_SET set );


/*****************************************************************************
** FUNCTION:   bcmOsTftpReadFile
**
** PURPOSE:    This function reads the specified file from the specified TFTP
**             server, placing the file data in the buffer.
**             Up to "bufferSize" bytes of the file will be read.
**
** PARAMETERS: serverIpAddress (I)  - the IP address of the TFTP server.
**             pFilename       (I)  - the fully qualified name of the file
**                                    to be read.
**             pBuffer         (O)  - buffer with retrieved data.
**             ifno            (I)  - IP Iface # over which the TFTP is performed
**             bufferSize      (I)  - Size of the buffer.
**             pBytesRead      (O)  - the number of bytes actually read.
**             abortFunc       (I)  - function to call to check if file read should
**                                    be aborted mid-stream.
**
** RETURNS:    0 if OK, error code - otherwise
**
** NOTE:
*****************************************************************************/
OS_STATUS bcmOsTftpReadFile(  IPADDR serverIpAddress,
                              char *pFilename,
                              UINT32 ifno,
                              char *pBuffer,
                              UINT32 bufferSize,
                              UINT32 *pBytesRead,
                              OS_ABORT_FNC abortFunc
                           );


/*****************************************************************************
** FUNCTION:   bcmOsDnsGetHostName
**
** PURPOSE:    Given an IP address, this function finds the corresponding host
**             name and returns it.
**
**
** PARAMETERS: ipAddr   [IN] - Specifies the address to be resolved to a name.
**             hostName [OUT] - Specifies the pointer to a buffer that is filled
**                               with the resolved name. The caller should
**                               preallocate the buffer to store the resolved name
**                               (which can be upto 255 characters).
**
** RETURNS:    OS_STATUS
**
*****************************************************************************/
OS_STATUS bcmOsDnsGetHostName( IPADDR ipAddr, char *hostName );


/***************************************************************************
** FUNCTION:   bcmOsDnsGetHostAddr
**
** PURPOSE:    Given the hostname, this function finds the corresponding IP
**             address and returns it.
**
** PARAMETERS: hostName [IN] - host name to resolve into an IP address
**             ipAddr   [OUT] - resolved IP address
**
** RETURNS:    OS_STATUS
**
*****************************************************************************/
OS_STATUS bcmOsDnsGetHostAddr( char *hostName, IPADDR *ipAddr );


/***************************************************************************
** FUNCTION:   bcmOsDnsGetHostAddrs
**
** PURPOSE:    Given the hostname, this function finds corresponding IP
**             addresses and returns them.
**
** PARAMETERS: hostName [IN]        - host name to resolve into IP addresses
**             ipAddr   [OUT]       - array of resolved IP addresses. Memory
**                                     must be allocated by caller.
**             numIpAddrs [IN/OUT]  - On input, points to the array length
**                                     of 'ipAddr'. On output, returns the
**                                     number of resolved IP addresses contained
**                                     in the 'ipAddr' array.
**             useCache [IN]        - indicates whether to use the DNS cache
**
** RETURNS:    OS_STATUS
**
*****************************************************************************/
OS_STATUS bcmOsDnsGetHostAddrs( char *hostName, IPADDR *ipAddr, int *numIpAddrs, BOOL useCache );

/****************************************************************************
** FUNCTION:   bcmOsDnsGetSRVName
**
** PURPOSE:    Retrieves the SRV record and returns target FQDN Name of server
**             providing requested service on requested domain.
**
** PARAMETERS: hostSRVName[IN]- host name and service with format
**                               "_kerberos._udp.broadcom.com\0"
**
**             srvtarget[OUT] - target servers providing service with format:
**                              "kerbserver1.broadcom.com.\0". If more than
**                              one server present, servers listed in order,
**                              "kerbserv1.broadcom.com.\0kerb2.broadcom.com.\0\0"
**
** RETURNS:    OS_STATUS
**
*****************************************************************************/
OS_STATUS bcmOsDnsGetSRVName( char *hostSRVName, char *srvtarget );


/****************************************************************************
** FUNCTION:   bcmOsStartDns
**
** PURPOSE:    Launches the domain name service (DNS).
**
** PARAMETERS: dnsAddr -  array of DNS server IP addresses
**             nServers - the number of DNS server IP addresses given in the
**                         list 'dnsAddr'. Must be less than OS_DNS_MAX_SERVERS.
**
** RETURNS:    OS_STATUS
**
*****************************************************************************/
OS_STATUS bcmOsStartDns( IPADDR *dnsAddr, int nServers, int ipstacknum );


#ifdef __cplusplus
    }
#endif

#endif  /* _BCMOS_H  */


